@@ -1,4 +1,4 @@
-// Copyright 2012 Luis Pedro Coelho <luis@luispedro.org>
+// Copyright 2012-2014 Luis Pedro Coelho <luis@luispedro.org>
 // License: MIT (see COPYING.MIT file)
 
 #ifndef LPC_TOOLS_H_INCLUDE_GUARD_WED_FEB__8_17_05_13_WET_2012
@@ -46,4 +46,31 @@ inline uint32_t read32_le(byte_source& s) {
     return (uint32_t(s1) << 16)|uint32_t(s0);
 }
 
+struct stack_based_memory_pool {
+    // This class manages an allocator which releases all allocated memory on
+    // stack exit
+    stack_based_memory_pool() { }
+    ~stack_based_memory_pool() {
+        for (unsigned i = 0; i != data_.size(); ++i) {
+            operator delete(data_[i]);
+            data_[i] = 0;
+        }
+    }
+
+    void* allocate(const int n) {
+        data_.reserve(data_.size() + 1);
+        void* d = operator new(n);
+        data_.push_back(d);
+        return d;
+    }
+
+    template <typename T>
+    T allocate_as(const int n) {
+        return static_cast<T>(this->allocate(n));
+    }
+
+    private:
+    std::vector<void*> data_;
+};
+
 #endif // LPC_TOOLS_H_INCLUDE_GUARD_WED_FEB__8_17_05_13_WET_2012
